/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package battle;

import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

/**
 *
 * @author jub
 * 
 * Provides collision detection utilities for World.
 */
public class CollisionDetector {

    private Rectangle screen;
    private Rectangle gridSection;
    private World world;
    private ArrayList<Element> limitsCollisions;
    private ArrayList<Element>[] preCollisions;
    private ArrayList<Element[]> collisions;
    private int gridL;
    private int gridH;

    /**Constructor, creates collision detector that interacts with world.
    @param _world World object in which collision are detected.
     */
    CollisionDetector(World _world) {
        world = _world;
        gridL = 10;
        gridH = 10;
        screen = new Rectangle(0, 0, World.WIDTH, World.HEIGHT);
        gridSection = new Rectangle(0, 0, World.WIDTH / gridL, World.HEIGHT / gridH);
        preCollisions = (ArrayList<Element>[]) new ArrayList[gridL * gridH];
        collisions = new ArrayList<Element[]>();
        limitsCollisions = new ArrayList<Element>();
    }

    /**Returns an ArrayList with an array of elements that collided with each other. */
    public ArrayList<Element[]> getCollisions() {
        return collisions;
    }

    /** Returns an ArrayList with Elements that collided with world limits.*/
    public ArrayList<Element> getLimitsCollisions() {
        return limitsCollisions;
    }

    /**Returns an array with elements on a certain area
    @param rect Rectangular area to check.
     */
    public ArrayList<Element> checkArea(Rectangle2D rect) {
        ArrayList<Element> eIA = new ArrayList<Element>();
        for (Element e : world.getUpdatedElements()) {
            if (e instanceof GElement) {
                GElement g = (GElement) e;
                if (rect.intersects(g.getBounds())) {
                    eIA.add(e);
                }
            }
        }
        return eIA;
    }

    /**Checks if element still exist in world.
    @param element element to check.
     */
    public boolean confirmExistence(Element element) {
        return world.elementPresent(element);
    }

    /**It starts collision detection in world.*/
    public void detectCollision() {
        checkScreenCollisions();
        if (world.getElements().size() > 1) {
            checkGrid();
            checkCollisions();
        }
    }

    /**Checks Collisions with world limits and stores elements in array.*/
    public void checkScreenCollisions() {
        limitsCollisions = new ArrayList<Element>();
        for (Element e : world.getUpdatedElements()) {
            if (e instanceof GElement) {
                GElement g = (GElement) e;
                if (!screen.contains(g.getBounds())) {
                    limitsCollisions.add(e);
                }
            }
        }
    }

    /**Checks grid for possible collisions between elements and stores them in array.*/
    public void checkGrid() {
        int val = 0;
        for (int i = 0; i < gridL; i++) {
            for (int j = 0; j < gridH; j++) {
                preCollisions[val] = new ArrayList<Element>();
                gridSection.setFrame(
                        i * World.WIDTH / gridL,
                        j * World.HEIGHT / gridH,
                        World.WIDTH / gridL,
                        World.HEIGHT / gridH);
                preCollisions[val] = checkArea(gridSection);
                //Remove isolated element's area.
                if (preCollisions[val].size() < 2) {
                    preCollisions[val] = new ArrayList<Element>();
                }
                val++;
            }
        }
    }

    /**Checks which possible collisions actually happen and stores them in array.*/
    public void checkCollisions() {
        collisions = new ArrayList<Element[]>();
        for (int i = 0; i < preCollisions.length; i++) {
            if (preCollisions[i].size() > 1) {
                for (int j = 0; j < preCollisions[i].size(); j++) {
                    for (int k = j + 1; k < preCollisions[i].size(); k++) {
                        Element e1 = preCollisions[i].get(j);
                        Element e2 = preCollisions[i].get(k);
                        if (e1 instanceof GElement && e2 instanceof GElement) {
                            GElement g1 = (GElement) e1;
                            GElement g2 = (GElement) e2;
                            if (g1.getBounds().intersects(g2.getBounds())) {
                                Element[] eA = new Element[2];
                                eA[0] = e1;
                                eA[1] = e2;
                                collisions.add(eA);
                            }
                        }
                    }
                }
            }
        }
        //Erase repeated elements.
        for (int j = 0; j < collisions.size(); j++) {
            for (int k = j + 1; k < collisions.size(); k++) {
                if (collisions.get(j)[0] == collisions.get(k)[0] && collisions.get(j)[1] == collisions.get(k)[1]) {
                    collisions.remove(k);
                    k--;
                }
            }
        }
    }
}
