/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package battle;

/**
 *
 * @author jub
 * 
 * Logic element that is contained in World.
 */
public class Element {

    protected static int id = 0;
    protected int eId = 0;
    protected float size;
    private Vector2D position;
    private Vector2D direction;
    protected int ang;
    private int rotationSpeed;
    private int speed;

    /**Constructor, creates a logic element.
    @param _position Initial position.
     * @param _size Object size.
     */
    public Element(Vector2D _position, float _size) {
        id++;
        eId = id;
        position = _position;
        ang = (int) (Math.random() * 360);
        direction = new Vector2D(Math.cos(ang * Math.PI / 180), Math.sin(ang * Math.PI / 180));
        size = _size;
        rotationSpeed = 1;
        speed = 1;
    }

    /**Returns identifier number.*/
    public int getId() {
        return eId;
    }

    /**Returns object size.*/
    public float getSize() {
        return size;
    }

    /**Returns direction angle.*/
    public int getAngle() {
        return ang;
    }

    /**Turns the object rotationSpeed degrees to the right.*/
    public void turnRight() {
        ang += rotationSpeed;
        ang %= 360;
        double radAng = ang * Math.PI / 180;
        direction.set(Math.cos(radAng), Math.sin(radAng));
    }

    /**Turns the object rotationSpeed degrees to the left.*/
    public void turnLeft() {
        if ((ang -= rotationSpeed) < 0) {
            ang = 359;
        }
        double radAng = ang * Math.PI / 180;
        direction.set(Math.cos(radAng), Math.sin(radAng));
    }

    /**Moves the object forward speed units.*/
    public void forward() {
        position = position.plus(direction.times(speed));
    }

    /**Moves the object back speed units.*/
    public void back() {
        position = position.minus(direction.times(speed));
    }

    /**Returns direction vector.*/
    public Vector2D getDir() {
        return direction;
    }

    /**Returns speed.*/
    public int getSpeed() {
        return speed;
    }

    /**Returns x position.*/
    public double getX() {
        return position.getX();
    }

    /**Returns y position.*/
    public double getY() {
        return position.getY();
    }

    /**Returns position vector.*/
    public Vector2D getPos() {
        return position;
    }

    /**Sets element speed.*/
    public void setSpeed(int _speed) {
        speed = _speed;
    }

    /**Sets element direction.*/
    public void setDir(Vector2D dir) {
        direction = dir;
    }
}
