/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package battle;

/**
 *
 * @author jub
 * This is the ship element. Its a space ship, and retarded at that.
 */
public class SElement extends GElement {

    public enum State {

        SEARCH, FOLLOW, ATTACK, TURNAWAY, STOPPED, AIM
    };

    private enum Movement {

        FORWARD, TURNLEFT, TURNRIGHT, BACK, SHOOT, FREE
    };
    private static int advance = 10;
    private static int rotation = 45;
    private static int followCount = 4;
    private State state;
    private Movement mov;
    private int movCount;
    private int backMovCount;
    private int rotCount;
    private int step;
    private boolean turned;
    private boolean shot;
    private final Gun gun;
    protected Sensor sensor;

    /**Constructor, creates space ship.
    @param _world World in which ship will be created.
     * @param position Initial ship position.
     * @param size Size of the ship to create.
     */
    public SElement(World _world, Vector2D position, float size) {
        super(position, size);
        state = State.SEARCH;
        mov = Movement.FREE;
        movCount = advance;
        backMovCount = advance;
        rotCount = rotation;
        step = 0;
        turned = false;
        sensor = new Sensor(_world, this);
        gun = new Gun(_world, this);
        shot = false;
    }

    /**Returns ship sensor*/
    public Sensor getSensor() {
        return sensor;
    }

    /**Tells the sensor to check if there's elements in sight.*/
    public void checkSenses() {
        sensor.sense();
    }

    /**Checks if a certain bullet was shot from this ship.*/
    public boolean shotBullet(Bullet bullet) {
        if (bullet.getId() == getId()) {
            return true;
        } else {
            return false;
        }
    }

    /**Updates ship behavior state.*/
    public void updateState() {
        switch (state) {
            case SEARCH:
                if (sensor.targetFound()) {
                    state = State.AIM;
                    reset();
                    break;
                }
                break;
            case AIM:
                if (sensor.traceLost()) {
                    state = State.FOLLOW;
                    reset();
                    break;
                }
                if (sensor.targetLocked()) {
                    state = State.ATTACK;
                    reset();
                    break;
                }
                if (sensor.targetDead()) {
                    state = State.SEARCH;
                    reset();
                    break;
                }
                break;
            case ATTACK:
                if (sensor.targetDead()) {
                    state = State.SEARCH;
                    reset();
                    break;
                }
                if (sensor.traceLost()) {
                    state = State.FOLLOW;
                    reset();
                    break;
                }
                if (shot) {
                    //SHOOTS
                    state = State.AIM;
                    reset();
                    break;
                }
                break;
            case FOLLOW:
                if (sensor.targetFound()) {
                    state = State.AIM;
                    reset();
                    break;
                }
                if (followLimitReached()) {
                    state = State.SEARCH;
                    reset();
                    break;
                }
                break;
            case TURNAWAY:
                if (turned) {
                    state = State.SEARCH;
                    reset();
                    break;
                }
                break;
            case STOPPED:
                state = State.TURNAWAY;
                break;
            default:
                state = State.SEARCH;
        }
    }
    
    /**Acts according to ship state.*/
    public void go() {
        if (mov == Movement.FREE) {
            switch (state) {
                case SEARCH:
                    mov = searchRoulette();
                    break;
                case AIM:
                    mov = aimMovement();
                    break;
                case FOLLOW:
                    mov = followMovement();
                    break;
                case ATTACK:
                    mov = attackMovement();
                    break;
                case TURNAWAY:
                    mov = turnAwayMovement();
                    break;
            }
        }
        if (eId == 1) {
//            System.out.print(eId + " : " + state + "\n");
        }
//        System.out.print(eId + " : "+mov+ "\n");
        move();
    }
    /**Resets the ship to initial behavioral values.*/
    public void reset() {
        movCount = advance;
        backMovCount = advance;
        rotCount = rotation;
        mov = Movement.FREE;
        step = 0;
        turned = false;
        followCount = 4;
    }

    /**Changes the state to stop machine at update and then resets values.*/
    public void stop() {
        state = State.STOPPED;
        reset();
    }

    /**Decides what to do while ship is aiming. */
    private Movement aimMovement() {
        //CLOCKWISE -> POSITIVO
        //C-CLOCKWISE -> NEGATIVO
//        System.out.print(eId + ": DEG = " + sensor.getTargetAngle() + "\n");
        rotCount = 1;
        if (sensor.getTargetAngle() > 0) {
            return Movement.TURNRIGHT;
        } else if (sensor.getTargetAngle() < 0) {
            return Movement.TURNLEFT;
        }
        return Movement.FREE;
    }

    /**Decides what to do while the ship is searching its sorroundings.*/
    private Movement searchRoulette() {
        switch ((int) (Math.random() * 6)) {
            case 0:
                return Movement.TURNLEFT;
            case 1:
                return Movement.TURNRIGHT;
            default:
                return Movement.FORWARD;
        }
    }

    /**Decides what to do while its turning away.*/
    private Movement turnAwayMovement() {
        switch (step++) {
            case 0:
                return Movement.BACK;
            case 1:
                return Movement.BACK;
            case 2:
                return Movement.TURNLEFT;
            case 3:
                return Movement.TURNLEFT;
            default:
                turned = true;
                return Movement.FORWARD;
        }
    }

    /**Decides that to do while its following a lead.*/
    private Movement followMovement() {
        return Movement.FORWARD;
    }

    /**Decides what to do at attacking.*/
    private Movement attackMovement() {
        shot = true;
        return Movement.SHOOT;
    }

    /**Ship action according to current ship movement.*/
    private void move() {
        switch (mov) {
            case FORWARD:
                forward();
                break;
            case BACK:
                back();
                break;
            case TURNLEFT:
                turnLeft();
                break;
            case TURNRIGHT:
                turnRight();
                break;
            case SHOOT:
                shoot();
                break;
            default:
                ;
        }
    }

    /**Returns true if ship has followed its las lead enough.*/
    private boolean followLimitReached() {
        if (followCount <= 0) {
            return true;
        } else {
            return false;
        }
    }

    /**Moves the graphic object a certain amount of times to complete a forward movement.*/
    @Override
    public void forward() {
        super.forward();
        if (movCount-- < 0) {
            followCount--;
            movCount = advance;
            mov = Movement.FREE;
        }
    }

    /**Moves the graphic object a certain amount of times to complete a turn to the left.*/
    @Override
    public void turnLeft() {
        super.turnLeft();
        if (rotCount-- < 0) {
            rotCount = rotation;
            mov = Movement.FREE;
        }
    }

    /**Moves the graphic object a certain amount of times to complete a turn to the right.*/
    @Override
    public void turnRight() {
        super.turnRight();
        if (rotCount-- < 0) {
            rotCount = rotation;
            mov = Movement.FREE;
        }

    }

    /**Moves the graphic object a certain amount of times to complete a back movement.*/
    @Override
    public void back() {
        super.back();
        if (backMovCount-- < 0) {
            backMovCount = advance;
            mov = Movement.FREE;
        }
    }

    /**Tells it's gun to fire a bullet to the world.*/
    public void shoot() {
        gun.fire();
    }
}
