/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package battle;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

/**
 *
 * @author jub
 * World which contains and makes element act according to its rules.
 */
public class World {

    public static int WIDTH = 1000;  // in pixels
    public static int HEIGHT = 700; // in pixels
    private ArrayList<Element> elements;
    private ArrayList<Element> elementsToAdd;
    private ArrayList<Element> elementsToRemove;
    private CollisionDetector collisionDetector;
    private MainPanel panel;

    /**Constructor, it initializes the necessary arrays.*/
    World() {
        elements = new ArrayList<Element>();
        elementsToAdd = new ArrayList<Element>();
        elementsToRemove = new ArrayList<Element>();
        collisionDetector = new CollisionDetector(this);
    }

    /**Returns actual elements in world.*/
    public ArrayList<Element> getElements() {
        return elements;
    }

    /**Returns a copy of elements.*/
    public ArrayList<Element> getUpdatedElements() {
        ArrayList<Element> updatedElement = new ArrayList<Element>();
        updatedElement.addAll(elements);
        return updatedElement;
    }

    /**Sets an element to remove at next update.*/
    public void postRemoveElement(Element e) {
        elementsToRemove.add(e);
    }

    /**Sets an element to be added at next update.*/
    public void postAddElement(Element e) {
        elementsToAdd.add(e);
    }

    /**Updates world state and element attributes.*/
    public void update() {
//        System.out.print("SIZE = " + elements.size() + "\n");
        ArrayList<Element> AuxElement = new ArrayList<Element>();
        AuxElement.addAll(elements);

        for (Element e : AuxElement) {
            if (e instanceof SElement) {
                SElement s = (SElement) e;
                s.checkSenses();
                s.updateState();
                s.go();
            }
            if (e instanceof Bullet) {
                Bullet b = (Bullet) e;
                b.shoot();
            }
        }
        elements.addAll(elementsToAdd);
        elements.removeAll(elementsToRemove);
        elementsToAdd.clear();
        elementsToRemove.clear();
        collisionDetector.detectCollision();
        processLimitsCollisions();
        processCollisions();

    }

    /**Links a panel to the world.
    @param _panel Panel to be linked.
     */
    public void setPanel(MainPanel _panel) {
        panel = _panel;
    }

    /**Decides what to draw through world.
    @param g2d Graphics2D object.
     */
    public void paintComponent(Graphics2D g2d) {
        ArrayList<Element> AuxElements = new ArrayList<Element>();
        AuxElements.addAll(elements);
        for (Element e : AuxElements) {
            if (e instanceof GElement) {
                GElement s = (GElement) e;
                s.draw(g2d);
            }
            if (e instanceof SElement) {
                SElement s = (SElement) e;
                s.getSensor().draw(g2d);
            }
        }
    }

    /**Adds an element to the world.
    @param _element Element to add.
     */
    public void addElement(Element _element) {
        elements.add(_element);
    }

    /**Check if an element is in specified coordinates.
    @param x X coordinate to check.
     * @param y Y coordinate to check.
     */
    public Element find(int x, int y) {
        for (Element e : elements) {
            if (e instanceof SElement) {
                SElement s = (SElement) e;
                if (s.contains(x, y)) {
                    return s;
                }
            }
        }
        return null;
    }

    /**It decides consequences of elements colliding with screen.*/
    public void processLimitsCollisions() {
        for (Element e : collisionDetector.getLimitsCollisions()) {
            if (e instanceof SElement) {
                SElement s = (SElement) e;
                s.stop();
            }
            if (e instanceof Bullet) {
                Bullet b = (Bullet) e;
                postRemoveElement(b);
            }
        }
    }

    /**It decides consequences of elements colliding with each other.*/
    public void processCollisions() {
        for (Element[] e : collisionDetector.getCollisions()) {
            if (e[0] != e[1] && e[0] instanceof GElement && e[1] instanceof GElement) {
                if (e[0] instanceof SElement && e[1] instanceof SElement) {
                    elements.remove(e[0]);
                    elements.remove(e[1]);
                } else if (e[0] instanceof Bullet && e[1] instanceof SElement) {
                    SElement s = (SElement) e[1];
                    Bullet b = (Bullet) e[0];
                    if (!s.shotBullet(b)) {
                        elements.remove(e[1]);
                        elements.remove(e[0]);
                    }
                } else if (e[1] instanceof Bullet && e[0] instanceof SElement) {
                    SElement s = (SElement) e[0];
                    Bullet b = (Bullet) e[1];
                    if (!s.shotBullet(b)) {
                        elements.remove(e[1]);
                        elements.remove(e[0]);
                    }
                } else if (e[1] instanceof Bullet && e[0] instanceof Bullet) {
                    Bullet b1 = (Bullet) e[0];
                    Bullet b2 = (Bullet) e[1];
                    if (b1.getId() != b2.getId()) {
                        elements.remove(e[1]);
                        elements.remove(e[0]);
                    }
                }
            }
        }
    }

    /**Checks if an elements is present in the world.
    @param element Element to be checked it's existence.
     */
    public boolean elementPresent(Element element) {
        return elements.contains(element);
    }

    /**Gets elements that are in a certain rectangular area.
    @param area Area to me checked.
     */
    public ArrayList<Element> getSensed(Rectangle2D area) {
        return collisionDetector.checkArea(area);
    }
}
